﻿/**
 * @author: Christof Rueß, http://hobbyelektronik.org
 * @license: http://creativecommons.org/licenses/by-nc-sa/3.0/
 * Feel free to share and remix this 
 * but:
 * - leave this comment in the code
 * - don't use this code for commercial purposes
 * - when you alter this code: c'mon, share it like I did!
 *
 * see: http://www.hobbyelektronik.org/wiki/index.php?title=Commercial_Use
 */
function ListView(container, columns, data, plugins, height) {
	if(typeof addCSSRule == "undefined") { alert("Module 'css' not included"); }
	//if(typeof DateFormat == "undefined") alert("Module 'DateStuff' not included");
	this._container = container;
	this.columns = columns;
	this.data = data;
	this._plugins = (typeof plugins == "object") ? plugins : [];
	this.csshash = "ListView_" + strPad(Math.floor(Math.random() * 1000000), 7, "0", "PAD_LEFT");
	this.table = {}; //container, table, colgroup, tbody
	this.height = height;
	
	//this.plugins = Array();
	
	this.columntypes = {
		"CHECKBOX" : { 
			"func" : function (data, column) {
				elem = document.createElement("input");
				data["field_" + column.field].lviData = elem;
				elem.type = "checkbox";
				elem.value = data[column.field];
				return elem;
			}
		},
		"DATE" : {
			"align" : "right",
			"func" : function(data, column) {
				if(column.formatter == undefined) column.formatter = new DateFormat("DD.MM.YYYY");
				elem = document.createTextNode(column.formatter.Format(new Date(data[column.field]*1000)));
				return elem;
			}
		},
		"DATETIME" : {
			"align" : "right",
			"func" : function(data, column) {
				if(column.formatter == undefined) column.formatter = new DateFormat("DD.MM.YYYY HH:mm");
				elem = document.createTextNode(column.formatter.Format(new Date(data[column.field]*1000)));
				return elem;
			}
		},
		"LIST" : { 
			"func" : function(data, column) {
				elem = document.createTextNode(data[column.field].join(", "));
				return elem;
			}
		},
		"IMAGE" : { 
			"func" : function(data, column) {
				elem = document.createElement("img");
				var img = data[column.field];
				if(img != "" && img != undefined) {
					elem.src = img;
				} else {
					elem = document.createTextNode("");
				}
				return elem;
			}
		},
		"IMAGELIST" : { 
			"func" : function(data, column) {
				elem = document.createElement("img");
				var img = column.imagelist[data[column.field]];
				if(img != "" && img != undefined) {
					elem.src = img;
				} else {
					elem = document.createTextNode("");
				}
			}
		},
		"TEXT" : {
			"func" : function(data, column, cell) {
				var content = data[column.field];
				
				if(typeof column.func == "function") {
					content = column.func(data, column, cell);
				}
				
				if(content === null || content === undefined) { content = ""; }
				return document.createTextNode(content);
			}
		},
		"NUMBER" : {
			"align" : "right",
			"func" : function(data, column, cell) {
				var content = data[column.field];
				if(typeof column.func == "function") {
					content = column.func(data, column, cell);
				}
				if(content === null || content === undefined) { content = ""; }
				return document.createTextNode(content);
			}
		},
		"ELEMENT" : {
			"func" : function(data, column, cell) {
				var content = data[column.field];
				
				if(typeof column.func == "function") {
					content = column.func(data, column, cell);
				}
				
				if(content === null || content === undefined) { content = document.createTextNode(""); }
				return content;
			}
		}
	};
	
	this._Construct();
}

ListView.prototype = {
	_Construct : function() {
		if(this._container === null) { return false; }
		
		this.table = {
			"table" : document.createElement("table"), 
			"colgroup" : document.createElement("colgroup"),
			"thead" : document.createElement("thead"),
			"tbody" : document.createElement("tbody")
		};
		
		if(this.height !== undefined) {
			this.table.tbody.style.height = (this.height - 16) + "px";
			this.table.tbody.style.overflowY = "scroll";
			this.table.tbody.style.overflowX = "hidden";
			this.columns[this.columns.length] = {title: "", type : "TEXT", func : function() {return ""; }, visible : true, width : 30};
		}
		
		//Select im IE vermeiden
		this.table.table.onselectstart = function(e) { return false; };
		
		this.table.table.tabIndex = '0';
		
		//this.table.table.style.width = '100px'; //wichtig für Firefox
		this.table.table.cellSpacing = "0"; //für IE
		this.table.table.className = "ListView3 " + this.csshash;
		this.table.table.appendChild(this.table.colgroup);
		this.table.table.appendChild(this.table.thead);
		this.table.table.appendChild(this.table.tbody);
		
		//this.table.tbody.style.height = "100px"
		
		this._container.appendChild(this.table.table);
		this._PopulateHead();
		this._InitPlugins();
		this._InitData();
		this.PopulateData();
	}, 
	
	_InitPlugins : function() {
		var plugin = null;
		for(var x = 0; x < this._plugins.length; x++) {
			plugin = this._plugins[x];
			plugin.Init(this);
		}
	},

	_PopulateHead : function() {
		//this._ClearHead();
		
		this.table.colgroup.colSpan = this.columns.length;
		var trfrag = document.createDocumentFragment();
		
		var thiscol;
		for(var x = 0; x < this.columns.length; x++) {
			thiscol = this.columns[x];
			if(thiscol.visible === false) { continue; }
			var col = document.createElement("col");
			if(typeof thiscol.width == "number" || typeof thiscol.width == "string") {
				col.width = this.columns[x].width; }
			this.table.colgroup.appendChild(col);
			
			thiscol.td = document.createElement("td");
			var coltitle = document.createElement("span");
			coltitle.appendChild(document.createTextNode(thiscol.title));
			coltitle.className = "coltitle";
			thiscol.td.appendChild(coltitle);
			thiscol.td.title = thiscol.title;
			
			thiscol.td.className = "col-" + x;
			trfrag.appendChild(thiscol.td);
			
			//funktioniert (noch) nicht mit IE
			//if(!document.all) {
				var css = addCSSRule("." + this.csshash + " .col-" + x);
				if(this.columntypes[thiscol.type] !== undefined && this.columntypes[thiscol.type].align !== undefined) {
					css.style.textAlign = this.columntypes[thiscol.type].align;
				} else {
					css.style.textAlign  = "left";
				}
			//}
			
		}
		var tr = document.createElement("tr");
		tr.appendChild(trfrag);
		this.table.thead.appendChild(tr);
		
		
		//Breite des Containers
		var contwidth = 0;
		for(x = 0; x < this.columns.length; x++) {
			contwidth += isNaN(this.columns[x].width) ? 100 : this.columns[x].width;
		}
		//contwidth += 16; //Scrollbar
		this.table.table.style.width = (contwidth-50) + "px";
	},
	
	_ClearHead : function() {
		while(this.table.colgroup.childNodes.length > 0) {
			this.table.colgroup.removeChild(this.tcols.colgroup.firstChild); }
		while(this.table.thead.childNodes.length > 0) {
			this.table.thead.removeChild(this.tcols.tbody.firstChild); }
	},
	
	_InitData : function() {
		/*this.table.colgroup.colSpan = this.columns.length
		
		var thiscol;
		for(var x = 0; x < this.columns.length; x++) {
			thiscol = this.columns[x];
			if(thiscol.visible == false) continue;
			var col = document.createElement("col");
			if(typeof thiscol.width == "number" || typeof thiscol.width == "string") 
				col.width = this.columns[x].width;
			this.tdata.colgroup.appendChild(col);
		}*/
		
	},
	
	PopulateData : function() {
		while(this.table.tbody.childNodes.length > 0) { this.table.tbody.removeChild(this.table.tbody.firstChild); }
		var tbodyfrag = document.createDocumentFragment();
		
		var thiscol;
		for(var y = 0; y < this.data.length; y++) {
			if(this.data[y].lviData === undefined) { this.data[y].lviData = {}; }
			var thisdata = this.data[y];
			var thisdataext = this.data[y].lviData;
			var tr = document.createElement("tr");
			
			//plugins
			for(var i = 0; i < this._plugins.length; i++) {
				if(typeof this._plugins[i].AddRow == "function") { this._plugins[i].AddRow(tr, this.columns, thisdata, y); }
			}
			
			for(var x = 0; x < this.columns.length; x++) {
				
				thiscol = this.columns[x];
				if(thiscol.visible === false) { continue; }
				var td = document.createElement("td");
				td.className = ("col-" + x);
				//td.appendChild(this._CreateCellContent(thisdata, thiscol, td));
				
				if(this.columntypes[thiscol.type] !== undefined) {
					td.appendChild(this.columntypes[thiscol.type].func(thisdata, thiscol, td));
				} else {
					td.appendChild(document.createTextNode("unknown cell type"));
				}
				
				//plugins
				for(i = 0; i < this._plugins.length; i++) {
					if(typeof this._plugins[i].AddCell == "function") { this._plugins[i].AddCell(td, thiscol, thisdata, y, x); }
				}
				
				tr.appendChild(td);
				this.data[y].lviData.trelem = tr;
			}
			tbodyfrag.appendChild(tr);
		}
		this.table.tbody.appendChild(tbodyfrag);
	},
	
	Refresh : function() {
		this.PopulateData();
	},
	
	AddRow : function(data, position) {
		if(position === null || position === undefined) { position = this.data.length; }
		this.data.splice(position, 0, data);
		this.Refresh();
		return position;
	},
	
	DelRow : function(rowid) {
		if(rowid > this.data.length) { rowid = this.data.length; }
		this.data.splice(rowid, 1);
		
		return this.data.length;
	},
	
	_CreateCellContent : function(data, column, cell) {
		if(this.columntypes[column.type] !== undefined) {
			return this.columntypes[column.type].func(data, column, cell);
		} else {
			return elem = document.createTextNode("unknown cell type");
		}
	},
	
	_getImgCache : function(file) {
		if(this.imgcache === undefined || this.imgcache === null) {
			this.imgcache = {"length": 0};
		}
		if(this.imgcache[file] === undefined) {
			var stylename = "imgcache_" + this.imgcache.length;
			this.imgcache.length++;
			
			this.imgcache[file] = stylename;
			
			var css = addCSSRule("." + this.csshash + " ." + stylename);
			var img = new Image();/*document.createElement("img");*/
			img.src = file;
			
			var putimg = function() {
				if(img.complete) {
					css.style.backgroundImage = "url('"+file+"')";
					css.style.width = img.width + "px";
					css.style.height = img.height + "px";
					window.clearInterval(tmr);
				}
			};
			var tmr = setInterval(putimg,10);
		}
		return this.imgcache[file];
	}
};


function LVSelectRow(multiple) {
	this.initialized = false;
	this.parent = null;
	this.multiple = (multiple === true) ? true : false;
	this.currentrow = null;
}

LVSelectRow.prototype = {
	Init : function(object) {
		if(this.initialized === true) { return; }
		this.initialized = true;
		this.parent = object;
		var oThis = this;
		
		//events
		this.parent.table.table.onkeydown = function(evt) {
			var el = (evt) ? evt.target : window.event.srcElement;
			var key = (evt) ? evt.keyCode : window.event.keyCode;
			
			if(oThis._HandleKeyPress(key, el) === true) { return; }
			if(window.event) { 
				window.event.cancelBubble = true; 
			} else { 
				evt.preventDefault(); 
				evt.stopPropagation();
			}
			return false;
		};
		
		//Parent Events
		this.parent.OnBeforeSelect = function(oldrow, newrow, evt) { return; };
		this.parent.OnSelect = function(rowid, colid, evt) { return; };
		this.parent.SelectRow = function(rowid, releaseevent, colid) { oThis.SelectRow(rowid, null, releaseevent === undefined ? true : releaseevent); };
		this.parent.OnDblClick = function(rowid, colid, evt) { return; };
		this.parent.GetSelectedItems = oThis.GetSelectedItems;
		this.parent.GetSelectedIndices = oThis.GetSelectedIndices;
	},
	
	GetSelectedItems : function() {
		//in parent's scope!
		var list = [];
		for(var i = 0; i < this.data.length; i++) {
			if(this.data[i].lviData.selected === true) {
				list[list.length] = this.data[i];
			}
		}
		return list;
	},
	
	GetSelectedIndices : function() {
		//in parent's scope!
		var list = [];
		for(var i = 0; i < this.data.length; i++) {
			if(this.data[i].lviData.selected === true) {
				list[list.length] = i;
			}
		}
		return list;
	},
	
	_HandleKeyPress : function(keycode, el) {
		var processkey = false;
		//FF releases the event on the table, IE on TD
		if(el !== null && el.nodeName != "TABLE" && el.nodeName != "TD") { return; }
		var newrow = this.currentrow;
		if(this.multiple === false) {
			switch(keycode) {
				case 38: //up
					if(newrow > 0) { newrow--; }
					this.SelectRow(newrow);
					break;
				case 40: //down
					if(newrow < this.parent.data.length-1) { newrow++; }
					this.SelectRow(newrow);
					break;
				case 36: //home
					newrow = 0;
					this.SelectRow(newrow);
					break;
				case 35: //end
					newrow = this.parent.data.length-1;
					this.SelectRow(newrow);
					break;
				default:
					processkey = true;
			}
		} else {
			switch(keycode) {
				case 38: //up
					if(this.currentrow > 0) { this.currentrow--; }
					break;
				case 40:
					if(this.currentrow < this.parent.data.length-1) { this.currentrow++; }
					break;
				case 36: //home
					this.currentrow = 0;
					break;
				case 35: //end
					this.currentrow = this.parent.data.length-1;
					break;
				case 32: //space
					this.SelectRow(this.currentrow);
					break;
				default:
					processkey = true;
			}
			this.parent.data[this.currentrow].lviData.trelem.focus();
		}
		return processkey;
	},
	
	//$.OnHeadAdd = function() {}
	
	AddRow : function(trelem, column, data, rowid) {
		var oThis = this;
		if(data.lviData.selected === true) { trelem.className += " selected"; }
		trelem.onclick = function(evt) { oThis.SelectRow(rowid, evt, true); };
		trelem.ondblclick = function(evt) { oThis.parent.OnDblClick(rowid, undefined, evt); };
	},
	
	SelectRow : function(rowid, evt, releaseevent) {
		if(releaseevent !== false) {
			if(this.parent.OnBeforeSelect(this.currentrow, rowid, evt) === false) { return; }
		}

		var el = (evt) ? evt.target : (window.event ? window.event.srcElement : null);
		//if(el != null && el.nodeName != "TR" && el.nodeName != "TD") return;

		var rows = this.parent.data;
		var rowelem;
		if(this.multiple === false) {
			for(var x = 0; x < rows.length; x++) {
				rowelem = rows[x].lviData;
				rowelem.selected = false;
				rowelem.trelem.className = rowelem.trelem.className.replace(/selected/, "");
				if(x == rowid) { 
					rowelem.trelem.className += " selected";
					rowelem.selected = true;
					this.currentrow = rowid;
				}
			}
		} else {
			
			/*if(evt != undefined && evt.shiftKey == true) {
				for(var x = ((this.currentrow<rowid) ? this.currentrow +1 : rowid);  ((this.currentrow>rowid) ? (x < this.currentrow +1) : (x < rowid+1)); (this.currentrow<rowid) ? x++ : x--) {
					cons ole.log(x);
					var dataext = this.parent.dataext[x];
					if(dataext.selected == true) {
						dataext.trelem.className = dataext.trelem.className.replace(/selected/, "");
						dataext.selected = false;
					} else {
						dataext.trelem.className += " selected";
						dataext.selected = true;
					}
				}
			} else {*/
				var dataext = this.parent.data[rowid].lviData;
				if(dataext.selected === true) {
					dataext.trelem.className = dataext.trelem.className.replace(/selected/, "");
					dataext.selected = false;
				} else {
					dataext.trelem.className += " selected";
					dataext.selected = true;
				}
			//}
		}
		this.currentrow = rowid;
		this.parent.OnSelect(this.currentrow, undefined, evt);
	}
	
	/*$.OnCellAdd = function(tdelem, column, data, dataext, rowid, colid) {
	
	}*/
};

function LVSortable() {
	this.initialized = false;
	this.parent = null;
	this.sortcolumns = [];
}

LVSortable.prototype = {
	Init : function(object) {
		this.initialized = true;
		this.parent = object;
		
		var oThis = this;
		for(var x = 0; x < this.parent.columns.length; x++) {
			this.parent.columns[x].td.onclick = function() {
				oThis.SelectCol(this);
			};
		}
		
	},
	
	SelectCol : function(e) {
		var columns = this.parent.columns;
		var selcolumn = null;
		for(var x = 0; x < columns.length; x++) {
			columns[x].td.className = columns[x].td.className.replace(/ sort(asc|desc)/, "");
			if(columns[x].td == e) { selcolumn = columns[x]; }
		}
		
		if(this.sortcolumns[0] === null) { this.sortcolumns = [{ "column" : null, "order" : null }]; }
		if(this.sortcolumns[0].column == selcolumn.field) {
			if(this.sortcolumns[0].order == "asc") {
				this.sortcolumns[0].order = "desc";
				e.className += " sortdesc";
				//e.style.width = (e.offsetWidth - 12) + "px"
				this.AddSortIcon(e);
			} else {
				this.sortcolumns[0].order = "asc";
				e.className += " sortasc";
				//e.style.width = (e.offsetWidth - 12) + "px"
				this.AddSortIcon(e);
			}
		} else {
			this.sortcolumns[0].column = selcolumn.field;
			this.sortcolumns[0].order = "desc";
			e.className += " sortdesc";
			//e.style.width = (e.offsetWidth - 12) + "px"
			this.AddSortIcon(e);
		}
		this.Sort();
	},
	
	AddSortIcon : function(e) {

	},
	
	Sort : function() {
		var oThis = this;
		
		var orderFunc2 = function(a, b) {
			
			var x = a[oThis.sortcolumns[0].column].toLowerCase();
			var y = b[oThis.sortcolumns[0].column].toLowerCase();
			var retval = (x < y) ? -1 : ((x > y) ? 1 : 0);
			
			if(oThis.sortcolumns[0].order.toLowerCase() == "asc") {
				retval *= -1;
			}
			
			return retval;
		};
		
		this.parent.data.sort(orderFunc2);
		this.parent.Refresh();
	}
};